package org.artifactory.release.bundle;

import org.apache.commons.lang3.tuple.Pair;
import org.artifactory.api.rest.distribution.bundle.models.ArtifactsBundleModel;
import org.artifactory.bundle.BundleTransactionStatus;
import org.artifactory.bundle.BundleType;
import org.artifactory.repo.RepoPath;
import org.artifactory.storage.db.bundle.model.DBArtifactsBundle;
import org.joda.time.DateTime;
import org.testng.annotations.Test;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.expectThrows;

/**
 * @author Eli Skoran 14/09/2020
 */

@Test
public class ReleaseBundleUtilsTest {
    private static final String BUNDLE_NAME = "bundle1";
    private static final String BUNDLE_VERSION = "version1";
    private static final String REPO_NAME = "release_bundles";

    @Test
    public void testBuildExportFileRepoPath() {
        String expectedRepoPath =
                REPO_NAME + "/" + BUNDLE_NAME + "/" + BUNDLE_VERSION + "/" + BUNDLE_NAME + "-" + BUNDLE_VERSION +
                        ".zip";

        RepoPath repoPath = ReleaseBundleUtils.buildExportFileRepoPath(BUNDLE_NAME, BUNDLE_VERSION, REPO_NAME);

        assertEquals(repoPath.toPath(), expectedRepoPath);
    }

    @Test
    public void testConvertToArtifactsBundleModel() {
        DBArtifactsBundle artifactsBundle = new DBArtifactsBundle();
        artifactsBundle.setName(BUNDLE_NAME);
        artifactsBundle.setVersion(BUNDLE_VERSION);
        artifactsBundle.setDateCreated(DateTime.now());
        artifactsBundle.setId(1111);
        artifactsBundle.setSignature("1q2w3e4r5t6y7u8i");
        artifactsBundle.setStatus(BundleTransactionStatus.COMPLETE);
        artifactsBundle.setStoringRepo(REPO_NAME);
        artifactsBundle.setType(BundleType.SOURCE);

        ArtifactsBundleModel artifactsBundleModel = ReleaseBundleUtils.convertToArtifactsBundleModel(artifactsBundle);

        assertEquals(artifactsBundleModel.getStatus(), artifactsBundle.getStatus());
        assertEquals(artifactsBundleModel.getStoringRepo(), artifactsBundle.getStoringRepo());
        assertEquals(artifactsBundleModel.getId(), artifactsBundle.getId());
        assertEquals(artifactsBundleModel.getName(), artifactsBundle.getName());
        assertEquals(artifactsBundleModel.getVersion(), artifactsBundle.getVersion());
        assertEquals(artifactsBundleModel.getType(), artifactsBundle.getType());

    }

    @Test
    public void testGenerateTransactionPath() {
        String transactionPath = ReleaseBundleUtils.generateTransactionPath(BUNDLE_NAME, BUNDLE_VERSION);

        assertEquals(transactionPath, "_intransit/" + BUNDLE_NAME + "/" + BUNDLE_VERSION);
    }

    @Test
    public void testExtractReleaseBundleNameAndVersion() {
        // wrong format
        expectThrows(IllegalArgumentException.class,
                () -> ReleaseBundleUtils.extractReleaseBundleNameAndVersion("badFormat")
        );

        // wrong format
        expectThrows(IllegalArgumentException.class,
                () -> ReleaseBundleUtils.extractReleaseBundleNameAndVersion("a/b#c")
        );


        Pair<String, String> nameVersion = ReleaseBundleUtils.extractReleaseBundleNameAndVersion("_intransit/b/c");
        assertEquals(nameVersion.getLeft(), "b");
        assertEquals(nameVersion.getRight(), "c");
    }
}
