package org.artifactory.opentracing;

import org.artifactory.api.events.EventLogHintManager;
import org.junit.Assert;
import org.junit.Test;
import org.mockito.Mockito;

import static org.artifactory.api.events.EventHintType.FederatedReplication;
import static org.artifactory.mirror.MirrorConstants.MIRROR_MARKER;

public class TraceableRunnableDecoratorTest {

    @Test
    public void testMirrorMarkerIsSetAndRemovedOnEachRunnable() {
        ArtifactoryTracer artifactoryTracer = Mockito.mock(ArtifactoryTracer.class);
        ServerTracingSpan currentSpan = Mockito.mock(ServerTracingSpan.class);
        Mockito.when(artifactoryTracer.getCurrentThreadActiveSpan()).thenReturn(currentSpan);
        TraceableRunnableDecorator runnableDecorator =
                new TraceableRunnableDecorator(assertMirrorMarker, artifactoryTracer);

        Assert.assertNull(EventLogHintManager.getHints());
        runnableDecorator.run();
        Assert.assertNull(EventLogHintManager.getHints());
    }

    private final Runnable assertMirrorMarker = () -> {
        Assert.assertNotNull(EventLogHintManager.getHints());
        Assert.assertTrue(EventLogHintManager.getHints().contains(FederatedReplication.getPrefix() + MIRROR_MARKER));
    };
}
